% =========================================================================
%> @file  Benchmark.m
%> @author		Christian Campestrini (c.campestrini@tum.de)
%> @brief		Calculates the benchmark score according to:
%>              Campestrini, C.; Horsche, M.F.; Zilberman, I.; Heil, T.;
%>              Zimmermann, T.; Jossen, A.: Validation and benchmark
%>              methods for battery management system functionalities -
%>              State of charge estimation algorithms
%>              In: Journal of Energy Storage 7, S. 3851, 2016
%>
%> @date		08.06.2016
%>
%> @param		Setup (settings for the plot and tables)
%> @param		FilterResult (estimation result struct)
%> @param		ProfileTiming (timing data for profile A for the drift
%>              calculation)
%>
%> @retval		none
%>
%> @syntax		Benchmark(Setup,FilterResult,ProfileTiming);
% =========================================================================
function Benchmark(Setup,FilterResult,ProfileTiming)
%% Define the Benchmark
    % Set scores and limits according to Equation 4
    Def.Score(1)=5;
    Def.Score(2)=4;
    Def.Score(3)=3;
    Def.Score(4)=2;
    Def.Score(5)=1;

    Def.ErrBound(1)=0.5/100;
    Def.ErrBound(2)=1/100;
    Def.ErrBound(3)=2/100;
    Def.ErrBound(4)=4/100;
    Def.ErrBound(5)=8/100;
    
    % Set reference time for the transient test
    Def.RefTime1=0.1;
    Def.RefTime2=0.2;
    
    FNrAlgo=fieldnames(FilterResult);

    ScoreProf.A=zeros(length(FNrAlgo),6);
    ScoreProf.B=zeros(length(FNrAlgo),6);
    ScoreProf.C=zeros(length(FNrAlgo),4);

    %% Algorithm loop
    for iAlgo=1:length(FNrAlgo)        
        %% Test loop
        FNrTest=fieldnames(FilterResult.(FNrAlgo{iAlgo}));
        for iTest=1:length(FNrTest)
            %% Temperatur loop
            FNrTemp=fieldnames(FilterResult.(FNrAlgo{iAlgo}).(FNrTest{iTest}));
            for iTemp=1:length(FNrTemp)
                %% Profile loop
                FNrProfile=fieldnames(FilterResult.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrTemp{iTemp}));
                for iProfile=1:length(FNrProfile)
                
                    % Init Data
                    EstimationAcc.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=0;
                    Gradient.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=0;
                    ResidualCharge.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=0;
                    Transient.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=0;    
                    TimeInErrBound=zeros(1,length(Def.ErrBound));
                    
                    Data=FilterResult.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile});
                    tstep=Data.EstTime(2)-Data.EstTime(1);

                    %% Calculate 10% of the Time for gradient validation
                    Def.T1EstTime=Data.EstTime(end)*Def.RefTime1;
                    Def.T2EstTime=Data.EstTime(end)*Def.RefTime2;
                    Def.InitEstSOC=Data.EstSOC(1);
                    Def.InitRefSOC=Data.RefSOC(1);
                    
                    %% Calculate estimation and residual charge error
                    Error.EstAcc=abs(Data.RefSOC-Data.EstSOC);              % Equation 5
                    Error.ResCha=abs(Data.ResChSOC-Data.EstSOC(end));       % Equation 9

                    %% Calculate gradient 
                    clear iRef iEst
                    % Find the constant current periods based on the timing data collected in function ProfileTiming
                    if ~strcmp(FNrProfile{iProfile}(1),'A')   
                        % For profile B and C the complete data can be used
                        iEst{1}=1:length(Data.EstTime);
                        iRef{1}=1:length(Data.RefTime);        
                    else
                        try
                            switch FNrTest{iTest}
                                % For the InitFail case the charge perido is not considered
                                case 'InitFail'   

                                    iEst{1}=find(Data.EstTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis1(1) & Data.EstTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis1(2));
                                    iEst{2}=find(Data.EstTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis2(1) & Data.EstTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis2(2));

                                    iRef{1}=find(Data.RefTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis1(1) & Data.RefTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis1(2));
                                    iRef{2}=find(Data.RefTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis2(1) & Data.RefTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis2(2));

                                otherwise
                                    iEst{1}=find(Data.EstTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeCha1(1) & Data.EstTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeCha1(2));
                                    iEst{2}=find(Data.EstTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis1(1) & Data.EstTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis1(2));
                                    iEst{3}=find(Data.EstTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis2(1) & Data.EstTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).EstTimeDis2(2));

                                    iRef{1}=find(Data.RefTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeCha1(1) & Data.RefTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeCha1(2));
                                    iRef{2}=find(Data.RefTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis1(1) & Data.RefTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis1(2));
                                    iRef{3}=find(Data.RefTime > ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis2(1) & Data.RefTime < ProfileTiming.(FNrTest{iTest}).(FNrTemp{iTemp}).(FNrProfile{iProfile}).RefTimeDis2(2));
                            end    
                        catch
                            error('Error: Timing data is not set correctly!')
                        end
                    end

                    GradientParts=zeros(1,length(iEst));
                    Error.Grad=[];
                    Fit=cell(length(iEst));
                    % Gradient calculation of the SOC error based on equation 7
                    for i=1:length(iEst)
                        
                        % Bring reference and estimation to the same size in case of minimal mismatch
                        if length(iRef{i})>length(iEst{i})
                            iRef{i}=iRef{i}(1:length(iEst{i}));
                        elseif length(iRef{i})<length(iEst{i})
                            iEst{i}=iEst{i}(1:length(iRef{i}));
                        end
                        
                        fSOC=Data.RefSOC(iRef{i})-Data.EstSOC(iEst{i});
                        fTime=Data.RefTime(iRef{i})-Data.RefTime(iRef{i}(1));
                        
                        % calculation of the linear regression 
                        Fit{i}=LinearRegression(fTime,fSOC-fSOC(1));                     
                        
                        % Different error calculation for the short term and long term test
                        if ~strcmp(FNrProfile{iProfile},'C')
                            Error.Grad(i)=abs(Fit{i}.p1*3600);              % short term
                        else
                            Error.Grad(i)=abs(Fit{i}.p1*3600*24*7);         % long term       
                        end
                        
                    end       
                    
                    clear iRef fSOC fTime Fit TimeOffset
                                  
                    %% Calculate the SOC error at 10% of the validation time based on equation 11
                    iTransient=find(Data.EstTime>=Def.T1EstTime);              
                    Error.SOCatEstTime=abs(Data.EstSOC(iTransient(1))-Data.RefSOC(iTransient(1))); 

                    %% Find error bound and the corresponding score
                    for i=1:length(Def.ErrBound)

                        % Calculation of the Estimation accuracy score based on equation 6
                        if i==1
                            TimeInErrBound(i)=tstep*length(Data.EstTime(Error.EstAcc <= Def.ErrBound(i))); 
                        else
                            TimeInErrBound(i)=tstep*length(Data.EstTime(Def.ErrBound(i-1) < Error.EstAcc & Error.EstAcc <= Def.ErrBound(i))); 
                        end          
                        
                        EstimationAcc.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=EstimationAcc.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)+Def.Score(i).*TimeInErrBound(i)/Data.EstTime(end);

                        % Calculation of the Drift score
                        if i==1
                            tmpIf=Error.Grad <= Def.ErrBound(i);
                        else
                            tmpIf=Error.Grad > Def.ErrBound(i-1) & Error.Grad <= Def.ErrBound(i);
                        end
                                  
                        for j=1:length(iEst)
                            if tmpIf(j)
                                GradientParts(j)=Def.Score(i);
                            end
                        end
                        
                        Gradient.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=mean(GradientParts);
                        
                        % Calculation of the Residual charge score
                        if i==1
                            tmpIf=Error.ResCha <= Def.ErrBound(i);
                        else
                            tmpIf=Error.ResCha > Def.ErrBound(i-1) & Error.ResCha <= Def.ErrBound(i);
                        end 
                        
                        if tmpIf
                            ResidualCharge.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=Def.Score(i);
                        end
                        
                        % Calculation of the Transient accuracy score based on equation 10
                        if i==1
                            tmpIf=Error.SOCatEstTime <= Def.ErrBound(i);
                        else
                            tmpIf=(Error.SOCatEstTime > Def.ErrBound(i-1)) & (Error.SOCatEstTime <= Def.ErrBound(i));
                        end                        
                        
                        if tmpIf
                            Transient.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp)=Def.Score(i)*abs(1-Def.InitEstSOC/Def.InitRefSOC);
                        end

                    end
                                        
                    clear tmpIf iEst TimeInErrBound GradientParts iTransient tstep
                    
                    %% Calculation of the failure stability (without mean) based on equation 12
                    if ~strcmp(FNrTest{iTest},'InitOk') && ~strcmp(FNrTest{iTest},'InitFail') && ~strcmp(FNrProfile{iProfile},'A2')
                        % When the InitOk value was 0, the failure stability score results to NaN
                        tmp.(FNrTest{iTest}).(FNrAlgo{iAlgo}).(FNrProfile{iProfile}).EstimationAccScore(:,iTemp)=NaN;                      
                        if EstimationAcc.(FNrAlgo{iAlgo}).InitOk.(FNrProfile{iProfile}).Score(iTemp)~=0
                            tmp.(FNrTest{iTest}).(FNrAlgo{iAlgo}).(FNrProfile{iProfile}).EstimationAccScore(:,iTemp)=Def.Score(1)-abs(EstimationAcc.(FNrAlgo{iAlgo}).InitOk.(FNrProfile{iProfile}).Score(iTemp)-EstimationAcc.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp));                                     
                        end
                        
                        tmp.(FNrTest{iTest}).(FNrAlgo{iAlgo}).(FNrProfile{iProfile}).ResidualChargeScore(:,iTemp)=NaN;
                        if ResidualCharge.(FNrAlgo{iAlgo}).InitOk.(FNrProfile{iProfile}).Score(iTemp)~=0
                            tmp.(FNrTest{iTest}).(FNrAlgo{iAlgo}).(FNrProfile{iProfile}).ResidualChargeScore(:,iTemp)=Def.Score(1)-abs(ResidualCharge.(FNrAlgo{iAlgo}).InitOk.(FNrProfile{iProfile}).Score(iTemp)-ResidualCharge.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp));                    
                        end

                        tmp.(FNrTest{iTest}).(FNrAlgo{iAlgo}).(FNrProfile{iProfile}).GradientScore(:,iTemp)=NaN;
                        if Gradient.(FNrAlgo{iAlgo}).InitOk.(FNrProfile{iProfile}).Score(iTemp)~=0
                            tmp.(FNrTest{iTest}).(FNrAlgo{iAlgo}).(FNrProfile{iProfile}).GradientScore(:,iTemp)=Def.Score(1)-abs(Gradient.(FNrAlgo{iAlgo}).InitOk.(FNrProfile{iProfile}).Score(iTemp)-Gradient.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp));                      
                        end
                    end
                    
                    %% Output of the values
                    tmp1= Transient.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp);
                    tmp2= EstimationAcc.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp);
                    tmp3= ResidualCharge.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp);
                    tmp4= Gradient.(FNrAlgo{iAlgo}).(FNrTest{iTest}).(FNrProfile{iProfile}).Score(iTemp);
                                       
                    if strcmp(FNrTest{iTest},'InitFail')
                        disp([FNrTest{iTest} ': ' FNrAlgo{iAlgo} '/' FNrProfile{iProfile} '/' FNrTemp{iTemp} ': ' sprintf('Transient: %.1f/5 | Estimation accuracy: %.1f/5 | Residual charge: %.1f/5 | Gradient: %.1f/5',tmp1,tmp2,tmp3,tmp4)])    
                    else
                        disp([FNrTest{iTest} ': ' FNrAlgo{iAlgo} '/' FNrProfile{iProfile} '/' FNrTemp{iTemp} ': ' sprintf('Transient: - | Estimation accuracy: %.1f/5 | Residual charge: %.1f/5 | Gradient: %.1f/5',tmp2,tmp3,tmp4)])    
                    end
                    
                    clear tmp1 tmp2 tmp3 tmp4
                    
                end
            end
        end
        
        clear iTest iTemp iProfile i j 
        %% Summerise data for tables and net diagram  
        % Estimation accuracy
        EstimationAccScore.(FNrAlgo{iAlgo}).A=mean(EstimationAcc.(FNrAlgo{iAlgo}).InitOk.A.Score);
        EstimationAccScore.(FNrAlgo{iAlgo}).B=mean(EstimationAcc.(FNrAlgo{iAlgo}).InitOk.B.Score);
        EstimationAccScore.(FNrAlgo{iAlgo}).C=EstimationAcc.(FNrAlgo{iAlgo}).InitOk.C.Score(end);

        % Gradient
        GradientScore.(FNrAlgo{iAlgo}).A=mean(Gradient.(FNrAlgo{iAlgo}).InitOk.A.Score);
        GradientScore.(FNrAlgo{iAlgo}).B=mean(Gradient.(FNrAlgo{iAlgo}).InitOk.B.Score);
        GradientScore.(FNrAlgo{iAlgo}).C=Gradient.(FNrAlgo{iAlgo}).InitOk.C.Score(end);
        
        % Rest Discharge  
        ResidualChargeScore.(FNrAlgo{iAlgo}).A=mean(ResidualCharge.(FNrAlgo{iAlgo}).InitOk.A.Score);
        ResidualChargeScore.(FNrAlgo{iAlgo}).B=mean(ResidualCharge.(FNrAlgo{iAlgo}).InitOk.B.Score);
        ResidualChargeScore.(FNrAlgo{iAlgo}).C=ResidualCharge.(FNrAlgo{iAlgo}).InitOk.C.Score(end);
        
        % Transient behaviour
        TransientScore.(FNrAlgo{iAlgo}).A=mean([Transient.(FNrAlgo{iAlgo}).InitFail.A.Score Transient.(FNrAlgo{iAlgo}).InitFail.A2.Score]);
        TransientScore.(FNrAlgo{iAlgo}).B=mean(Transient.(FNrAlgo{iAlgo}).InitFail.B.Score);

        % Temperature stability based on equation 14 and 15
        TempStabilityScore.(FNrAlgo{iAlgo}).A.Est=5-2*std(EstimationAcc.(FNrAlgo{iAlgo}).InitOk.A.Score,1);      
        TempStabilityScore.(FNrAlgo{iAlgo}).A.Res=5-2*std(ResidualCharge.(FNrAlgo{iAlgo}).InitOk.A.Score,1); 
        TempStabilityScore.(FNrAlgo{iAlgo}).A.Gra=5-2*std(Gradient.(FNrAlgo{iAlgo}).InitOk.A.Score,1);    
        TempStabilityScore.(FNrAlgo{iAlgo}).A.Tra=5-2*std(Transient.(FNrAlgo{iAlgo}).InitFail.A.Score,1);    

        TempStabilityScore.(FNrAlgo{iAlgo}).B.Est=5-2*std(EstimationAcc.(FNrAlgo{iAlgo}).InitOk.B.Score,1);      
        TempStabilityScore.(FNrAlgo{iAlgo}).B.Res=5-2*std(ResidualCharge.(FNrAlgo{iAlgo}).InitOk.B.Score,1); 
        TempStabilityScore.(FNrAlgo{iAlgo}).B.Gra=5-2*std(Gradient.(FNrAlgo{iAlgo}).InitOk.B.Score,1);    
        TempStabilityScore.(FNrAlgo{iAlgo}).B.Tra=5-2*std(Transient.(FNrAlgo{iAlgo}).InitFail.B.Score,1);

        TempStabilityScore.(FNrAlgo{iAlgo}).A2.Tra=5-2*std(Transient.(FNrAlgo{iAlgo}).InitFail.A2.Score,1);  
        
        % Calculates the mean of all temperatures based on equation 16
        TempStabilityScore.(FNrAlgo{iAlgo}).AllA=mean([TempStabilityScore.(FNrAlgo{iAlgo}).A.Tra; TempStabilityScore.(FNrAlgo{iAlgo}).A.Est; TempStabilityScore.(FNrAlgo{iAlgo}).A.Gra; TempStabilityScore.(FNrAlgo{iAlgo}).A.Res]);
        TempStabilityScore.(FNrAlgo{iAlgo}).AllB=mean([TempStabilityScore.(FNrAlgo{iAlgo}).B.Tra; TempStabilityScore.(FNrAlgo{iAlgo}).B.Est; TempStabilityScore.(FNrAlgo{iAlgo}).B.Gra; TempStabilityScore.(FNrAlgo{iAlgo}).B.Res]);

        % Error Stability, calculates the mean off all offsets based on equation 12
        P={'A','B','C'};
        for iP=1:3
            ErrorStability.(FNrAlgo{iAlgo}).(P{iP}).Est=mean([tmp.IOffset.(FNrAlgo{iAlgo}).(P{iP}).EstimationAccScore; tmp.UOffset.(FNrAlgo{iAlgo}).(P{iP}).EstimationAccScore; tmp.ROffset.(FNrAlgo{iAlgo}).(P{iP}).EstimationAccScore]);
            ErrorStability.(FNrAlgo{iAlgo}).(P{iP}).Res=mean([tmp.IOffset.(FNrAlgo{iAlgo}).(P{iP}).ResidualChargeScore; tmp.UOffset.(FNrAlgo{iAlgo}).(P{iP}).ResidualChargeScore; tmp.ROffset.(FNrAlgo{iAlgo}).(P{iP}).ResidualChargeScore]);
            ErrorStability.(FNrAlgo{iAlgo}).(P{iP}).Gra=mean([tmp.IOffset.(FNrAlgo{iAlgo}).(P{iP}).GradientScore; tmp.UOffset.(FNrAlgo{iAlgo}).(P{iP}).GradientScore; tmp.ROffset.(FNrAlgo{iAlgo}).(P{iP}).GradientScore]);   
        end
        
        % Calculates the mean of all categories based on equation 13
        ErrorStabilityMean.(FNrAlgo{iAlgo}).A=mean([ErrorStability.(FNrAlgo{iAlgo}).A.Est; ErrorStability.(FNrAlgo{iAlgo}).A.Res; ErrorStability.(FNrAlgo{iAlgo}).A.Gra]);
        ErrorStabilityMean.(FNrAlgo{iAlgo}).B=mean([ErrorStability.(FNrAlgo{iAlgo}).B.Est; ErrorStability.(FNrAlgo{iAlgo}).B.Res; ErrorStability.(FNrAlgo{iAlgo}).B.Gra]);
        ErrorStabilityMean.(FNrAlgo{iAlgo}).C=mean([ErrorStability.(FNrAlgo{iAlgo}).C.Est; ErrorStability.(FNrAlgo{iAlgo}).C.Res; ErrorStability.(FNrAlgo{iAlgo}).C.Gra]);       
        
        % Calculates the mean of all error stability scores
        ErrorStabilityTot.(FNrAlgo{iAlgo}).A=mean(ErrorStabilityMean.(FNrAlgo{iAlgo}).A(~isnan(ErrorStabilityMean.(FNrAlgo{iAlgo}).A)));
        ErrorStabilityTot.(FNrAlgo{iAlgo}).B=mean(ErrorStabilityMean.(FNrAlgo{iAlgo}).B(~isnan(ErrorStabilityMean.(FNrAlgo{iAlgo}).B)));
        ErrorStabilityTot.(FNrAlgo{iAlgo}).C=ErrorStabilityMean.(FNrAlgo{iAlgo}).C(end); 
        
        % All scores a collected
        ScoreProf.A(iAlgo,:)=[EstimationAccScore.(FNrAlgo{iAlgo}).A ResidualChargeScore.(FNrAlgo{iAlgo}).A TransientScore.(FNrAlgo{iAlgo}).A GradientScore.(FNrAlgo{iAlgo}).A ErrorStabilityTot.(FNrAlgo{iAlgo}).A TempStabilityScore.(FNrAlgo{iAlgo}).AllA];
        ScoreProf.B(iAlgo,:)=[EstimationAccScore.(FNrAlgo{iAlgo}).B ResidualChargeScore.(FNrAlgo{iAlgo}).B TransientScore.(FNrAlgo{iAlgo}).B GradientScore.(FNrAlgo{iAlgo}).B ErrorStabilityTot.(FNrAlgo{iAlgo}).B TempStabilityScore.(FNrAlgo{iAlgo}).AllB];
        ScoreProf.C(iAlgo,:)=[EstimationAccScore.(FNrAlgo{iAlgo}).C ResidualChargeScore.(FNrAlgo{iAlgo}).C GradientScore.(FNrAlgo{iAlgo}).C ErrorStabilityTot.(FNrAlgo{iAlgo}).C];
        
        % The tables are created and saved
        TabScore(EstimationAcc,ErrorStability,TempStabilityScore,FNrAlgo{iAlgo},Setup.Path);
        TabScore(ResidualCharge,ErrorStability,TempStabilityScore,FNrAlgo{iAlgo},Setup.Path);
        TabScore(Gradient,ErrorStability,TempStabilityScore,FNrAlgo{iAlgo},Setup.Path);
        TabScore(Transient,ErrorStability,TempStabilityScore,FNrAlgo{iAlgo},Setup.Path);
    end
    
    clear iAlgo 
    
    % Saving of the benchmark results
    save([Setup.Path '\Quality.mat'],'ScoreProf')
    
    % Plot the net diagramm 
    clear handles
    handles.Fig=figure('name',Setup.FigureName);
    set(handles.Fig,...
        'Units','centimeters',...
        'Position',[0 0 8 21]);        % Set figure size
    
    handles=NetPlot(handles,Setup,ScoreProf);          % Plot figure           
    NetStyle(handles,Setup,FNrAlgo);                   % Style figure
    
    if Setup.SaveFigures==1
        saveas(handles.Fig,[Setup.Path Setup.FigureName '.fig'])     % Save figure
    end    

end